@@ -1,13 +1,13 @@
-# $Id: TLPSRC.pm 30037 2013-04-19 15:48:28Z karl $
+# $Id: TLPSRC.pm 39054 2015-12-08 21:47:09Z karl $
 # TeXLive::TLPSRC.pm - module for handling tlpsrc files
-# Copyright 2007-2013 Norbert Preining
+# Copyright 2007-2014 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
 package TeXLive::TLPSRC;
 
 use FileHandle;
-use TeXLive::TLConfig qw($CategoriesRegexp);
+use TeXLive::TLConfig qw($CategoriesRegexp $DefaultCategory);
 use TeXLive::TLUtils;
 use TeXLive::TLPOBJ;
 use TeXLive::TLTREE;
@@ -15,7 +15,7 @@ use TeXLive::TLTREE;
 my $_tmp;
 my %autopatterns;  # computed once internally
 
-my $svnrev = '$Revision: 30037 $';
+my $svnrev = '$Revision: 39054 $';
 my $_modulerevision;
 if ($svnrev =~ m/: ([0-9]+) /) {
   $_modulerevision = $1;
@@ -106,7 +106,7 @@ sub from_file {
     # (blank lines are significant in tlpobj, but not tlpsrc)
 
     if ($line =~ /^ /) {
-      die "$srcfile: non-continuation indentation not allowed: `$line'";
+      die "$srcfile:$lineno: non-continuation indentation not allowed: `$line'";
     }
     # remove terminal white space
     $line =~ s/\s+$//;
