@@ -1,17 +1,13 @@
-# $Id: TLPaper.pm 37974 2015-07-28 06:42:13Z preining $
+# $Id: TLPaper.pm 39129 2015-12-17 11:34:14Z preining $
 # TeXLive::TLPaper.pm - query/modify paper sizes for our various programs
-# Copyright 2008-2014 Norbert Preining
+# Copyright 2008-2015 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 #
-# TODO TODO TODO
-# if no paperconfig line is found in paper_do_simple it adds to the end of the
-# file the new line, but this does not work on context because it 
-# has an \endinput at the end ... needs to be fixed
 
 package TeXLive::TLPaper;
 
-my $svnrev = '$Revision: 37974 $';
+my $svnrev = '$Revision: 39129 $';
 my $_modulerevision;
 if ($svnrev =~ m/: ([0-9]+) /) {
   $_modulerevision = $1;
@@ -37,7 +33,7 @@ BEGIN {
 
 =head1 NAME
 
-C<TeXLive::TLPaper> -- paper size setting
+C<TeXLive::TLPaper> -- TeX Live paper size setting module
 
 =head1 SYNOPSIS
 
@@ -302,8 +298,10 @@ sub paper_xdvi {
 # pdftex pdftexconfig.tex format
 # /--- pdftexconfig.tex ---
 # |...
-# |\pdfpagewidth=NNN true <unit>
-# |\pdfpageheight=NNN true <unit>
+# |\ifx\pdfpagewidth\undefined\else \pdfpagewidth=210 true mm \fi
+# |\ifx\pagewidth\undefined\else \pagewidth=210 true mm \fi
+# |\ifx\pdfpageheight\undefined\else \pdfpageheight=297 true mm \fi
+# |\ifx\pageheight\undefined\else \pageheight=297 true mm \fi
 # |...
 # \------------------------
 #
@@ -324,20 +322,29 @@ sub paper_pdftex {
   my @lines = <FOO>;
   close(FOO);
 
+  my @cpwidx;
+  my @cphidx;
   my ($cpw, $cph);
-  my ($cpwidx, $cphidx);
   my $endinputidx;
   # read the lines and the last pdfpageswidth/height wins
   for my $idx (0..$#lines) {
     my $l = $lines[$idx];
-    if ($l =~ m/^\s*\\pdfpagewidth\s*=?\s*(.+)\s*$/) {
-      $cpw = $1;
-      $cpwidx = $idx;
+    if ($l =~ m/\\(pdf|luatex|)pagewidth\s*=?\s*([0-9.,]+\s*true\s*[^\s]*)/) {
+      if (defined($cpw) && $cpw ne $2) {
+        tl_warn("TLPaper: inconsistent paper sizes in $inp for page width! Please fix that.\n");
+        return $F_ERROR;
+      }
+      $cpw = $2;
+      push @cpwidx, $idx;
       next;
     }
-    if ($l =~ m/^\s*\\pdfpageheight\s*=?\s*(.+)\s*$/) {
-      $cph = $1;
-      $cphidx = $idx;
+    if ($l =~ m/\\(pdf|luatex|)pageheight\s*=?\s*([0-9.,]+\s*true\s*[^\s]*)/) {
+      if (defined($cph) && $cph ne $2) {
+        tl_warn("TLPaper: inconsistent paper sizes in $inp for page height! Please fix that.\n");
+        return $F_ERROR;
+      }
+      $cph = $2;
+      push @cphidx, $idx;
       next;
     }
     if ($l =~ m/^\s*\\endinput\s*/) {
@@ -382,26 +389,40 @@ sub paper_pdftex {
         }
       }
       if ($found) {
-        my $newwline = 
-          '\pdfpagewidth=' . ${$pdftex_papersize{$newpaper}}[0] . "\n";
-        my $newhline = 
-          '\pdfpageheight=' . ${$pdftex_papersize{$newpaper}}[1] . "\n";
-        if (defined($cpwidx)) {
-          $lines[$cpwidx] = $newwline;
+        my $newwidth = ${$pdftex_papersize{$newpaper}}[0];
+        my $newheight = ${$pdftex_papersize{$newpaper}}[1];
+        if (@cpwidx) {
+          for my $idx (@cpwidx) {
+            ddebug("TLPaper: before line: $lines[$idx]");
+            ddebug("TLPaper: replacement: $newwidth\n");
+            $lines[$idx] =~ s/pagewidth\s*=\s*[0-9.,]+\s*true\s*[^\s]*/pagewidth=$newwidth/;
+            ddebug("TLPaper: after line : $lines[$idx]");
+          }
         } else {
+          my $addlines = '\ifx\pdfpagewidth\undefined\else \pdfpagewidth=' . $newwidth . ' \fi' . "\n";
+          $addlines .= '\ifx\luatexpagewidth\undefined\else \luatexpagewidth=' . $newwidth . ' \fi' . "\n";
+          $addlines .= '\ifx\pagewidth\undefined\else \pagewidth=' . $newwidth . ' \fi' . "\n";
           if (defined($endinputidx)) {
-            $lines[$endinputidx] = $newwline . $lines[$endinputidx];
+            $lines[$endinputidx] = $addlines . $lines[$endinputidx];
           } else {
-            $lines[$#lines] = $newwline;
+            $lines[$#lines] = $addlines;
           }
         }
-        if (defined($cphidx)) {
-          $lines[$cphidx] = $newhline;
+        if (@cphidx) {
+          for my $idx (@cphidx) {
+            ddebug("TLPaper: before line: $lines[$idx]");
+            ddebug("TLPaper: replacement: $newheight\n");
+            $lines[$idx] =~ s/pageheight\s*=\s*[0-9.,]+\s*true\s*[^\s]*/pageheight=$newheight/;
+            ddebug("TLPaper: after line : $lines[$idx]");
+          }
         } else {
+          my $addlines = '\ifx\pdfpageheight\undefined\else \pdfpageheight=' . $newheight . ' \fi' . "\n";
+          $addlines .= '\ifx\luatexpageheight\undefined\else \luatexpageheight=' . $newheight . ' \fi' . "\n";
+          $addlines .= '\ifx\pageheight\undefined\else \pageheight=' . $newheight . ' \fi' . "\n";
           if (defined($endinputidx)) {
-            $lines[$endinputidx] = $newhline . $lines[$endinputidx];
+            $lines[$endinputidx] = $addlines . $lines[$endinputidx];
           } else {
-            $lines[$#lines] = $newwline;
+            $lines[$#lines] = $addlines;
           }
         }
         info("$0: setting paper size for pdftex to $newpaper.\n");
@@ -620,6 +641,109 @@ sub paper_dvipdfmx {
 sub paper_context {
   my $outtree = shift;
   my $newpaper = shift;
+  my ($outcomp, $filecomp) = setup_names("context");
+  my $dftfile = $paper{'context'}{'default_file'};
+  my $outfile = "$outtree/$outcomp/$filecomp";
+  my $inp = &find_paper_file("context", "tex", $filecomp, "cont-sys.rme", $dftfile);
+
+  return($F_ERROR) unless $inp; 
+
+  open(FOO, "<$inp") || die "$0: open($inp) failed: $!";
+  my @lines = <FOO>;
+  close(FOO);
+
+  my @idx;
+  my $idxlast;
+  my $endinputidx = -1;
+  my $currentpaper;
+  # read the lines and the last setuppapersize before the endinput wins
+  for my $idx (0..$#lines) {
+    my $l = $lines[$idx];
+    if ($l =~ m/^[^%]*\\endinput/) {
+      $endinputidx = $idx;
+      last;
+    }
+    if ($l =~ m/^\s*\\setuppapersize\s*\[([^][]*)\].*$/) {
+      if (defined($currentpaper) && $currentpaper ne $1) {
+        tl_warn("TLPaper: inconsistent paper sizes in $inp! Please fix that.\n");
+        return $F_ERROR;
+      }
+      $currentpaper = $1;
+      $idxlast = $idx;
+      push @idx, $idx;
+      next;
+    }
+  }
+  # if we haven't found a paper line, assume a4
+  $currentpaper || ($currentpaper = "a4");
+  # trying to find the right papersize
+  #
+  if (defined($newpaper)) {
+    if ($newpaper eq "--list") {
+      info("$currentpaper\n");
+      for my $p (keys %pdftex_papersize) {
+        info("$p\n") unless ($p eq $currentpaper);
+      }
+    } elsif ($newpaper eq "--returnlist") {
+      my @ret = ();
+      push @ret, "$currentpaper";
+      for my $p (keys %pdftex_papersize) {
+        push @ret, $p unless ($p eq $currentpaper);
+      }
+      return \@ret;
+    } else {
+      my $found = 0;
+      for my $p (keys %pdftex_papersize) {
+        if ($p eq $newpaper) {
+          $found = 1;
+          last;
+        }
+      }
+      if ($found) {
+        if (@idx) {
+          for my $idx (@idx) {
+            ddebug("TLPaper: before line: $lines[$idx]");
+            ddebug("TLPaper: replacement: $newpaper\n");
+            $lines[$idx] =~ s/setuppapersize\s*\[([^][]*)\]\[([^][]*)\]/setuppapersize[$newpaper][$newpaper]/;
+            ddebug("TLPaper: after line : $lines[$idx]");
+          }
+        } else {
+          my $addlines = "\\setuppapersize[$newpaper][$newpaper]\n";
+          if (defined($endinputidx)) {
+            $lines[$endinputidx] = $addlines . $lines[$endinputidx];
+          } else {
+            $lines[$#lines] = $addlines;
+          }
+        }
+        info("$0: setting paper size for context to $newpaper.\n");
+        mkdirhier(dirname($outfile));
+        # if we create the outfile we have to call mktexlsr
+        TeXLive::TLUtils::announce_execute_actions("files-changed")
+          unless (-r $outfile);
+        if (!open(TMP, ">$outfile")) {
+          tlwarn("$0: Cannot write to $outfile: $!\n");
+          tlwarn("Not setting paper size for context.\n");
+          return($F_ERROR);
+        }
+        for (@lines) { print TMP; }
+        close(TMP) || warn "$0: close(>$outfile) failed: $!";
+        TeXLive::TLUtils::announce_execute_actions("regenerate-formats");
+        # TODO should we return the value of announce_execute action?
+        return($F_OK);
+      } else {
+        tlwarn("$0: Not a valid paper size for context: $newpaper\n");
+        return($F_WARNING);
+      }
+    }
+  } else {
+    info("Current context paper size (from $inp): $currentpaper\n");
+  }
+  return($F_OK);
+}
+
+sub paper_context_old {
+  my $outtree = shift;
+  my $newpaper = shift;
 
   my ($outcomp, $filecomp) = setup_names("context");
   my $dftfile = $paper{'context'}{'default_file'};
@@ -630,11 +754,11 @@ sub paper_context {
   my @sizes = keys %pdftex_papersize;
   # take care here, the \\\\ are necessary in some places and not in 
   # some others because there is no intermediate evaluation
-  return &paper_do_simple($inp, "context", '^\s*\\\\setuppapersize\s*', 
-            '^\s*\\\\setuppapersize\s*\[([^][]*)\].*$',
+  return &paper_do_simple($inp, "context", '^\s*%?\s*\\\\setuppapersize\s*', 
+            '^\s*%?\s*\\\\setuppapersize\s*\[([^][]*)\].*$',
             sub {
               my ($ll,$np) = @_;
-              if ($ll =~ m/^\s*\\setuppapersize\s*/) {
+              if ($ll =~ m/^\s*%?\s*\\setuppapersize\s*/) {
                 return("\\setuppapersize[$np][$np]\n");
               } else {
                 return($ll);
